
' =============== Telemarketing Blocker 20X2.bas ===============

' 	This program runs on a PICAXE-20X2.
' 	It detects an incoming telephone ring signal, siezes the phone line, 
'	plays a message asking the caller to press "1". After a "1" is received 
'	it places a ring signal on the phone line and plays a ring sound on a local speaker.

' === Constants & Variables ===

symbol LEDActive = A.0			' LED indicating circuit is active
symbol RingIn    = pinC.0		' incoming ring detected on this pin
symbol RingOut   = C.1			' outgoing ring tone on this pin
symbol PlayIt    = C.2			' a high on this pin plays the message
symbol ADCTime   = C.3			' the time set POT is read on this pin
symbol LED1hr    = C.4			' the 1 hr time delay LED is on this pin
symbol LED2hr    = C.5			' the 2 hr time delay LED is on this pin
symbol SetBtn    = C.6			' the button to set time delay and activate
symbol ADCLine   = C.7			' this pin monitors phone line voltage via U7
symbol K1	     = B.0			' K1 coil control pin
symbol K2	     = B.1			' K2 coil control pin
symbol K3	     = B.2			' K3 coil control pin
symbol K4	     = B.3			' K4 coil control pin
symbol Tone1     = pinB.4		' 697  Hz tone detected signal in on this pin
symbol Tone2     = pinB.5		' 1209 Hz tone detected signal in on this pin
symbol LED4hr    = B.6			' the 4 hr time delay LED is on this pin
symbol LEDNoEnd  = B.7			' the infinite time delay LED is on this pin
symbol index     = W0			' index used for counting	
symbol i	     = W1			' minutes time counter
symbol k	     = W2			' microsecond time counter		
symbol t	     = b7			' hours time counter
symbol ADCVal    = b8			' variable to store ADC readings			' 					
symbol delay     = b10			' variable to store delay time
symbol Rpt       = b11			' button press repeat counter

' === Directives ===

#com 3					' specify serial port
#picaxe 20X2				' specify processor

' ===Begin Program ===

dirsC = %00110110				' set C.1,C.2,C.4,C.5 pins to output
dirsB = %11001111				' set B.4,B.5 pins to input
high K4					' turn on relay K4
gosub CheckOnTime				' go to sub to check turn off time delay setting

CheckForRing:				' sub to check fo incoming ring signal

high LEDActive				' turn on circuit active LED

do

if RingIN = 1 then gosub main		' keep checking for incoming ring indication on pin C.0
pause 1					' pause 1 millisecond
gosub TimeExpired				' go check turn off timer

loop


main:						

high K1					' turn on relay K1, seize phone line
pause 100
high K2					' turn on relay K2, connect audio circuits
pause 1000
high PlayIt					' play ISD1820 message

for index = 1 to 2000			' count for 20 seconds
if Tone1 = 0 and Tone2 = 0 then gosub UKRing	' if ring detected go generate outgoing ring
pause 10					' pause 10 millisecs
next index
goto HangUp					' go to hangup sub on time out

CheckOnTime:				' sub to read turn off delay time pot setting			

do

readadc C.3,ADCVal			' read ADC voltage from POT R9
pause 100
if ADCVal < 64 then let delay = 1 				' interpret ADC readings < 64 as 1hr delay
high LED1hr Low LED2hr Low LED4Hr Low LEDNoEnd endif	' turn on 1hr delay LED
if ADCVal > 64 and ADCVal < 128 then let delay = 2 	' interpret ADC readings > 64 and < 128 as 2hr delay
high LED2hr Low LED1hr Low LED4Hr Low LEDNoEnd endif	' turn on 2hr delay LED
if ADCVal > 128 and ADCVal <192 then let delay = 4 	' interpret ADC readings > 128 and < 192 as 4hr delay
high LED4hr Low LED1hr Low LED2Hr Low LEDNoEnd endif	' turn on 4hr delay LED
if ADCVal > 192 then let delay = 0 				' interpret ADC readings > 192 as endless delay
high LEDNoEnd Low LED1hr Low LED2Hr Low LED4hr endif	' turn on no end delay LED

gosub CheckTimeSet						' go to CheckTimeSet sub 

loop

CheckTimeSet:							' sub to check if delay time has been set

button SetBtn, 0, 1, 0, Rpt, 1, CheckForRing		' if Set button is pressed goto CheckForRing sub
return

TimeExpired:							' sub to count time and check if time expired
inc k if k = 10000 then inc i let k = 0 endif		' increment K. If K = 10000 (~1 minute) inc minutes count i
if i = 60 then inc t let i = 0 endif			' if minutes counter = 60, inc hours counter t
if delay > 0 and t = delay then low K4 endif		' if time count equals pre-set delay turn off power (K4)
return


UKRing:								' a sub routine to generate the UK ring sound

high K3								' turn on relay K3, connect outgoing ring to line & spkr

low Playit								' ensure ISD1820 off
pause 100

for index = 1 to 10						' generate sound for 10 cycles
  
sound C.1, (100, 40)						' ring tone for 400ms
pause 400								' wait 400ms
sound C.1, (100, 40)						' ring tone for 400ms
pause 2000								' wait 2000ms
gosub PickUp							' go to PickUp sub
next index
goto HangUp								' go to HangUp sub on time out
return

PickUp:								' a sub to check if the home phone was picked up

readadc C.7, ADCVal						' read the voltage on pin C.7
pause 100
if ADCVal >120 then goto ClearRelays			' if phone picked up ADC > 120 go clear relays
return

ClearRelays:							' a sub to turn off relays not required after phone pickup
low K2 low K3 low playit low LEDActive			' turn off K2, K3 and Active LED off
pause 100

do
readadc C.7, ADCVal						' read the voltage on pin C.7
pause 100
if ADCVal <120 then low K1 pause 1000 goto CheckForRing endif ' if phone hung up ADC < 120 turn off relay K1 (drop line)
loop

HangUp:								' a sub to simulate hanging up the phone
low K1 low K2 low K3 low playit low LEDActive		' deactivate all relays, ISD 1820 and Active LED
pause 100
high K1 pause 500 low K1 pause 500				' cycle K1
high K1 pause 500 low K1 pause 500				' cycle K1
goto CheckForRing							' hangup complete, go check for new incoming ring
end